Package: mi
Type: Package
Title: Missing Data Imputation and Model Checking
Version: 1.2
Date: 2025-09-01
Authors@R: c(person("Andrew", "Gelman", email = "gelman@stat.columbia.edu", role = "ctb"), 
             person("Jennifer", "Hill", email = "jennifer.hill@nyu.edu", role = "ctb"),
             person("Yu-Sung", "Su", email = "suyusung@tsinghua.edu.cn", role = c("aut")), 
             person("Masanao", "Yajima", email = "my2167@columbia.edu", role = "ctb"), 
             person("Maria", "Pittau", email = "grazia@stat.columbia.edu", role = "ctb"), 
             person("Ben", "Goodrich", email = "benjamin.goodrich@columbia.edu", role = c("cre", "aut")), 
             person("Yajuan", "Si", email = "sophie2012@gmail.com", role = "ctb"),
             person("Jon", "Kropko", email = "jkropko@gmail.com", role = "aut"))
Description: The mi package provides functions for data manipulation, imputing missing values in an approximate Bayesian framework, diagnostics of the models used to generate the imputations, confidence-building mechanisms to validate some of the assumptions of the imputation algorithm, and functions to analyze multiply imputed data sets with the appropriate degree of sampling uncertainty.
VignetteBuilder: knitr
Depends: R (>= 3.0.0), methods, Matrix, stats4
Imports: arm (>= 1.4-11)
Suggests: betareg, lattice, knitr, MASS, nnet, parallel, sn, survival,
        truncnorm, foreign
URL: https://sites.stat.columbia.edu/gelman/
License: GPL (>= 2)
LazyLoad: yes
Author: Andrew Gelman [ctb],
  Jennifer Hill [ctb],
  Yu-Sung Su [aut],
  Masanao Yajima [ctb],
  Maria Pittau [ctb],
  Ben Goodrich [cre, aut],
  Yajuan Si [ctb],
  Jon Kropko [aut]
Maintainer: Ben Goodrich <benjamin.goodrich@columbia.edu>
NeedsCompilation: no
Packaged: 2025-09-01 20:01:42 UTC; ben
Repository: CRAN
Date/Publication: 2025-09-02 07:50:02 UTC
Built: R 4.6.0; ; 2026-04-26 11:28:29 UTC; unix
