#exportPattern("^[[:alpha:]]+")

importFrom(arm, display, fround, pfround, traceplot, bayesglm.fit, bayespolr)
importFrom(graphics, hist)
importFrom(Matrix, image, Matrix)
importFrom(stats4, coef, plot, summary, vcov)
import(methods)

importFrom("grDevices", "colorRampPalette", "grey", "heat.colors", "n2mfrow")
importFrom("graphics", "abline", "axis", "barplot", "lines", "par", "points", "segments", "title")
importFrom("stats", "as.dendrogram", "as.formula", "binomial", "cor",
           "cov", "cov2cor", "dbinom", "dist", "dlogis", "dnorm",
           "gaussian", "hclust", "is.ts", "lm", "lowess",
           "mahalanobis", "make.link", "median", "model.frame",
           "model.matrix", "model.response", "na.omit", "optim",
           "order.dendrogram", "pcauchy", "plogis", "pnorm", "poisson",
           "predict", "pt", "qlogis", "qnorm", "qpois", "quantile",
           "quasibinomial", "quasipoisson", "rbeta", "rbinom",
           "rchisq", "rgamma", "rmultinom", "rnbinom", "rnorm",
           "rpois", "rt", "runif", "sd", "uniroot", "var")
importFrom("utils", "head", "tail", "write.table")


exportClassPattern("^[[:alpha:]]+")
exportMethods(change, change_family, change_link, change_model, change_imputation_method, 
              change_size, change_transformation, change_type,
              complete, fit_model, get_parameters, hist, image, 
              mi, missing_data.frame, missing_variable, plot)
export(display, mi2BUGS, mi2stata, mipply, multinomial, pool, 
       .prune, .possible_missing_variable, Rhats, rdata.frame)

S3method(as.double, missing_variable)
S3method(as.double, categorical)
S3method(as.double, continuous)
S3method(as.double, count)
S3method(as.double, irrelevant)
S3method(as.double, missing_data.frame)
S3method(as.data.frame, missing_data.frame)
S3method(dim, missing_data.frame)
S3method(dimnames, missing_data.frame)
S3method(names, missing_data.frame)
S3method(dim, mi)
S3method(dimnames, mi)
S3method(names, mi)
S3method(is.na, missing_variable)
S3method(is.na, missing_data.frame)
S3method(is.na, mi)
S3method(length, missing_variable)
S3method(length, missing_data.frame)
S3method(length, mi)
S3method(print, mdf_list)
S3method(print, mi_list)
S3method("[", missing_data.frame)
S3method("[<-", missing_data.frame)
S3method("[[", missing_data.frame)
S3method("[[<-", missing_data.frame)
S3method("$", missing_data.frame)
S3method("$<-", missing_data.frame)
