#!/usr/bin/env bash
# @Project      : tql-cloudml.${PACKAGE_NAME}
# @File         : ${NAME}
# @Time         : 2019-06-05 12:13
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : ${DESCRIPTION}
cloudml tensorboard create -n tql-tb -l fds://browser-algo-nanjing/TensorBoard
