#!/usr/bin/env bash
# @Project      : tql-cloudml
# @Time         : 2019-07-01 15:56
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     :p PyCharm
# @Description  : ${DESCRIPTION}

#a=$(python p.py)
#
#echo $a

dt=$(date "+%Y%m%d")
echo "time: $dt"