#!/usr/bin/env bash
#cloudml quota_v2 update -p guaranteed -c 32 -M 64G -e yuanjie@xiaomi.com \
#-g 1 \
#-gt v100

#cloudml quota_v2 update -p best-effort \
#-gc 16 -c 256 -M 512 \
#-e yuanjie@xiaomi.com

#cloudml quota_v2 update -p guaranteed \
#-g 1 -c 128 -M 128 \
#-gt v100-16g -gt p4-8g \
#-gc 1 -gc 2 \
#-e yuanjie@xiaomi.com


#cloudml dev save-v2 -f master mydev



